@@ -62,6 +62,9 @@ representing a floating-point number: [1, -1][sign]*man*2**exp where
 sign is 0 or 1 and bc should correspond to the number of bits used to
 represent the mantissa (man) in binary notation, e.g.
 
+Explanation
+===========
+
 >>> from sympy.core.evalf import bitcount
 >>> sign, man, exp, bc = 0, 5, 1, 3
 >>> n = [1, -1][sign]*man*2**exp
@@ -82,7 +85,10 @@ if the corresponding complex part is None.
 def fastlog(x):
     """Fast approximation of log2(x) for an mpf value tuple x.
 
-    Notes: Calculated as exponent + width of mantissa. This is an
+    Explanation
+    ===========
+
+    Calculated as exponent + width of mantissa. This is an
     approximation for two reasons: 1) it gives the ceil(log2(abs(x)))
     value and 2) it is too high by 1 in the case that x is an exact
     power of 2. Although this is easy to remedy by testing to see if
@@ -117,6 +123,9 @@ def pure_complex(v, or_real=False):
     a and b are Numbers, else None. If `or_real` is True then 0 will
     be returned for `b` if `v` is a real number.
 
+    Examples
+    ========
+
     >>> from sympy.core.evalf import pure_complex
     >>> from sympy import sqrt, I, S
     >>> a, b, surd = S(2), S(3), sqrt(2)
@@ -429,7 +438,7 @@ def add_terms(terms, prec, target_prec):
     Helper for evalf_add. Adds a list of (mpfval, accuracy) terms.
 
     Returns
-    -------
+    =======
 
     - None, None if there are no non-zero terms;
     - terms[0] if there is only 1 term;
@@ -1060,7 +1069,10 @@ def evalf_integral(expr, prec, options):
 
 def check_convergence(numer, denom, n):
     """
-    Returns (h, g, p) where
+    Returns
+    =======
+
+    (h, g, p) where
     -- h is:
         > 0 for convergence of rate 1/factorial(n)**h
         < 0 for divergence of rate factorial(n)**(-h)
@@ -1541,6 +1553,9 @@ def N(x, n=15, **options):
     r"""
     Calls x.evalf(n, \*\*options).
 
+    Explanations
+    ============
+
     Both .n() and N() are equivalent to .evalf(); use the one that you like better.
     See also the docstring of .evalf() for information on the options.
 
