@@ -43,9 +43,11 @@ Refrences
 """
 
 if cin:
-    from sympy.codegen.ast import (Variable, IntBaseType, FloatBaseType, String,
-        Integer, Float, FunctionPrototype, FunctionDefinition, FunctionCall,
-        none, Return, Assignment, Type)
+    from sympy.codegen.ast import (Variable, Integer, Float,
+        FunctionPrototype, FunctionDefinition, FunctionCall,
+        none, Return, Assignment, intc, int8, int16, int64,
+        uint8, uint16, uint32, uint64, float32, float64, float80,
+        aug_assign, bool_, While, CodeBlock)
     from sympy.codegen.cnodes import (PreDecrement, PostDecrement,
         PreIncrement, PostIncrement)
     from sympy.core import Add, Mod, Mul, Pow, Rel
@@ -89,6 +91,29 @@ if cin:
             """Initializes the code converter"""
             super(CCodeConverter, self).__init__()
             self._py_nodes = []
+            self._data_types = {
+                "void": {
+                    cin.TypeKind.VOID: none
+                },
+                "bool": {
+                    cin.TypeKind.BOOL: bool_
+                },
+                "int": {
+                    cin.TypeKind.SCHAR: int8,
+                    cin.TypeKind.SHORT: int16,
+                    cin.TypeKind.INT: intc,
+                    cin.TypeKind.LONG: int64,
+                    cin.TypeKind.UCHAR: uint8,
+                    cin.TypeKind.USHORT: uint16,
+                    cin.TypeKind.UINT: uint32,
+                    cin.TypeKind.ULONG: uint64
+                },
+                "float": {
+                    cin.TypeKind.FLOAT: float32,
+                    cin.TypeKind.DOUBLE: float64,
+                    cin.TypeKind.LONGDOUBLE: float80
+                }
+            }
 
         def parse(self, filenames, flags):
             """Function to parse a file with C source code
@@ -211,8 +236,7 @@ if cin:
             Returns
             =======
 
-            A variable node as Declaration, with the given value or 0 if the
-            value is not provided
+            A variable node as Declaration, with the initial value if given
 
             Raises
             ======
@@ -223,10 +247,34 @@ if cin:
             Notes
             =====
 
-            This function currently only supports basic Integer and Float data
-            types
+            The function currently supports following data types:
+
+            Boolean:
+                bool, _Bool
+
+            Integer:
+                8-bit: signed char and unsigned char
+                16-bit: short, short int, signed short,
+                    signed short int, unsigned short, unsigned short int
+                32-bit: int, signed int, unsigned int
+                64-bit: long, long int, signed long,
+                    signed long int, unsigned long, unsigned long int
+
+            Floating point:
+                Single Precision: float
+                Double Precision: double
+                Extended Precision: long double
 
             """
+            if node.type.kind in self._data_types["int"]:
+                type = self._data_types["int"][node.type.kind]
+            elif node.type.kind in self._data_types["float"]:
+                type = self._data_types["float"][node.type.kind]
+            elif node.type.kind in self._data_types["bool"]:
+                type = self._data_types["bool"][node.type.kind]
+            else:
+                raise NotImplementedError("Only bool, int "
+                    "and float are supported")
             try:
                 children = node.get_children()
                 child = next(children)
@@ -238,86 +286,50 @@ if cin:
                     child = next(children)
 
                 val = self.transform(child)
-                # List in case of variable assignment,
-                # FunctionCall node in case of a function call
-                if (child.kind == cin.CursorKind.INTEGER_LITERAL
-                    or child.kind == cin.CursorKind.UNEXPOSED_EXPR):
-                    if (node.type.kind == cin.TypeKind.INT):
-                        type = IntBaseType(String('integer'))
-                        # when only one decl_ref_expr is assigned
-                        # e.g., int b = a;
-                        if isinstance(val, str):
-                            value = Symbol(val)
-                        # e.g., int b = true;
-                        elif isinstance(val, bool):
+
+                supported_rhs = [
+                    cin.CursorKind.INTEGER_LITERAL,
+                    cin.CursorKind.FLOATING_LITERAL,
+                    cin.CursorKind.UNEXPOSED_EXPR,
+                    cin.CursorKind.BINARY_OPERATOR,
+                    cin.CursorKind.PAREN_EXPR,
+                    cin.CursorKind.UNARY_OPERATOR,
+                    cin.CursorKind.CXX_BOOL_LITERAL_EXPR
+                ]
+
+                if child.kind in supported_rhs:
+                    if isinstance(val, str):
+                        value = Symbol(val)
+                    elif isinstance(val, bool):
+                        if node.type.kind in self._data_types["int"]:
                             value = Integer(0) if val == False else Integer(1)
-                        # when val is integer or character literal
-                        # e.g., int b = 1; or int b = 'a';
-                        elif isinstance(val, (Integer, int, Float, float)):
-                            value = Integer(val)
-                        # when val is combination of both of the above
-                        # but in total only two nodes on rhs
-                        # e.g., int b = a * 1;
-                        else:
-                            value = val
-
-                    elif (node.type.kind == cin.TypeKind.FLOAT):
-                        type = FloatBaseType(String('real'))
-                        # e.g., float b = a;
-                        if isinstance(val, str):
-                            value = Symbol(val)
-                        # e.g., float b = true;
-                        elif isinstance(val, bool):
+                        elif node.type.kind in self._data_types["float"]:
                             value = Float(0.0) if val == False else Float(1.0)
-                        # e.g., float b = 1.0;
-                        elif isinstance(val, (Integer, int, Float, float)):
+                        elif node.type.kind in self._data_types["bool"]:
+                            value = sympify(val)
+                    elif isinstance(val, (Integer, int, Float, float)):
+                        if node.type.kind in self._data_types["int"]:
+                            value = Integer(val)
+                        elif node.type.kind in self._data_types["float"]:
                             value = Float(val)
-                        # e.g., float b = a * 1.0;
-                        else:
-                            value = val
-
-                    elif (node.type.kind == cin.TypeKind.BOOL):
-                        type = Type(String('bool'))
-                        # e.g., bool b = a;
-                        if isinstance(val, str):
-                            value = Symbol(val)
-                        # e.g., bool b = 1;
-                        elif isinstance(val, (Integer, int, Float, float)):
+                        elif node.type.kind in self._data_types["bool"]:
                             value = sympify(bool(val))
-                        # e.g., bool b = a * 1;
-                        else:
-                            value = val
-
                     else:
-                        raise NotImplementedError("Only bool, int "
-                            "and float are supported")
-
-                elif (child.kind == cin.CursorKind.CALL_EXPR):
-                    return Variable(
-                        node.spelling
-                    ).as_Declaration(
                         value = val
+
+                    return Variable(
+                    node.spelling
+                    ).as_Declaration(
+                        type = type,
+                        value = value
                     )
 
-                # when val is combination of more than two expr and
-                # integer(or float) i.e. it has binary operator
-                # Or var decl has parenthesis in the rhs(as a parent Clang node)
-                # Or it has unary operator
-                # Or it has boolean literal on rhs i.e. true or false
-                elif (child.kind == cin.CursorKind.BINARY_OPERATOR
-                    or child.kind == cin.CursorKind.PAREN_EXPR
-                    or child.kind == cin.CursorKind.UNARY_OPERATOR
-                    or child.kind == cin.CursorKind.CXX_BOOL_LITERAL_EXPR):
-                    if (node.type.kind == cin.TypeKind.INT):
-                        type = IntBaseType(String('integer'))
-                    elif (node.type.kind == cin.TypeKind.FLOAT):
-                        type = FloatBaseType(String('real'))
-                    elif (node.type.kind == cin.TypeKind.BOOL):
-                        type = Type(String('bool'))
-                    else:
-                        raise NotImplementedError("Only bool, int "
-                            "and float are supported")
-                    value = val
+                elif child.kind == cin.CursorKind.CALL_EXPR:
+                    return Variable(
+                        node.spelling
+                        ).as_Declaration(
+                            value = val
+                        )
 
                 else:
                     raise NotImplementedError("Given "
@@ -329,26 +341,11 @@ if cin:
                         ))
 
             except StopIteration:
-
-                if (node.type.kind == cin.TypeKind.INT):
-                    type = IntBaseType(String('integer'))
-                    value = Integer(0)
-                elif (node.type.kind == cin.TypeKind.FLOAT):
-                    type = FloatBaseType(String('real'))
-                    value = Float(0.0)
-                elif (node.type.kind == cin.TypeKind.BOOL):
-                    type = Type(String('bool'))
-                    value = false
-                else:
-                    raise NotImplementedError("Only bool, int "
-                            "and float are supported")
-
-            return Variable(
+                return Variable(
                 node.spelling
-            ).as_Declaration(
-                type = type,
-                value = value
-            )
+                ).as_Declaration(
+                    type = type
+                )
 
         def transform_function_decl(self, node):
             """Transformation Function For Function Declaration
@@ -365,16 +362,18 @@ if cin:
 
 
             """
-            token = node.get_tokens()
-            c_ret_type = next(token).spelling
-            if (c_ret_type == 'void'):
-                ret_type = none
-            elif(c_ret_type == 'int'):
-                ret_type = IntBaseType(String('integer'))
-            elif (c_ret_type == 'float'):
-                ret_type = FloatBaseType(String('real'))
+
+            if node.result_type.kind in self._data_types["int"]:
+                ret_type = self._data_types["int"][node.result_type.kind]
+            elif node.result_type.kind in self._data_types["float"]:
+                ret_type = self._data_types["float"][node.result_type.kind]
+            elif node.result_type.kind in self._data_types["bool"]:
+                ret_type = self._data_types["bool"][node.result_type.kind]
+            elif node.result_type.kind in self._data_types["void"]:
+                ret_type = self._data_types["void"][node.result_type.kind]
             else:
-                raise NotImplementedError("Variable not yet supported")
+                raise NotImplementedError("Only void, bool, int "
+                    "and float are supported")
             body = []
             param = []
             try:
@@ -441,12 +440,15 @@ if cin:
             ValueError if multiple children encountered in the parameter node
 
             """
-            if (node.type.kind == cin.TypeKind.INT):
-                type = IntBaseType(String('integer'))
-                value = Integer(0)
-            elif (node.type.kind == cin.TypeKind.FLOAT):
-                type = FloatBaseType(String('real'))
-                value = Float(0.0)
+            if node.type.kind in self._data_types["int"]:
+                type = self._data_types["int"][node.type.kind]
+            elif node.type.kind in self._data_types["float"]:
+                type = self._data_types["float"][node.type.kind]
+            elif node.type.kind in self._data_types["bool"]:
+                type = self._data_types["bool"][node.type.kind]
+            else:
+                raise NotImplementedError("Only bool, int "
+                    "and float are supported")
             try:
                 children = node.get_children()
                 child = next(children)
@@ -459,10 +461,15 @@ if cin:
 
                 # If there is a child, it is the default value of the parameter.
                 lit = self.transform(child)
-                if (node.type.kind == cin.TypeKind.INT):
+                if node.type.kind in self._data_types["int"]:
                     val = Integer(lit)
-                elif (node.type.kind == cin.TypeKind.FLOAT):
+                elif node.type.kind in self._data_types["float"]:
                     val = Float(lit)
+                elif node.type.kind in self._data_types["bool"]:
+                    val = sympify(bool(lit))
+                else:
+                    raise NotImplementedError("Only bool, int "
+                        "and float are supported")
 
                 param = Variable(
                     node.spelling
@@ -474,12 +481,11 @@ if cin:
                 param = Variable(
                     node.spelling
                 ).as_Declaration(
-                        type = type,
-                        value = value
+                    type = type
                 )
 
             try:
-                value = self.transform(next(children))
+                self.transform(next(children))
                 raise ValueError("Can't handle multiple children on parameter")
             except StopIteration:
                 pass
@@ -744,6 +750,25 @@ if cin:
             """
             return self.transform(next(node.get_children()))
 
+        def transform_compound_assignment_operator(self, node):
+            """Transformation function for handling shorthand operators
+
+            Returns
+            =======
+
+            augmented_assignment_expression: Codegen AST node
+                    shorthand assignment expression represented as Codegen AST
+
+            Raises
+            ======
+
+            NotImplementedError
+                If the shorthand operator for bitwise operators
+                (~=, ^=, &=, |=, <<=, >>=) is encountered
+
+            """
+            return self.transform_binary_operator(node)
+
         def transform_unary_operator(self, node):
             """Transformation function for handling unary operators
 
@@ -826,7 +851,8 @@ if cin:
 
             # supported operators list
             operators_list = ['+', '-', '*', '/', '%','=',
-            '>', '>=', '<', '<=', '==', '!=', '&&', '||']
+            '>', '>=', '<', '<=', '==', '!=', '&&', '||', '+=', '-=',
+            '*=', '/=', '%=']
 
             # this stack will contain variable content
             # and type of variable in the rhs
@@ -893,6 +919,13 @@ if cin:
                         raise NotImplementedError(
                             "Bitwise operator has not been "
                             "implemented yet!")
+
+                    # token is a shorthand bitwise operator
+                    elif token.spelling in ['&=', '|=', '^=', '<<=',
+                    '>>=']:
+                        raise NotImplementedError(
+                            "Shorthand bitwise operator has not been "
+                            "implemented yet!")
                     else:
                         raise NotImplementedError(
                             "Given token {} is not implemented yet!"
@@ -935,7 +968,7 @@ if cin:
 
         def priority_of(self, op):
             """To get the priority of given operator"""
-            if op == '=':
+            if op in ['=', '+=', '-=', '*=', '/=', '%=']:
                 return 1
             if op in ['&&', '||']:
                 return 2
@@ -977,6 +1010,8 @@ if cin:
                 return [Or(as_Boolean(lhs_value), as_Boolean(rhs_value)), 'expr']
             if op == '=':
                 return [Assignment(Variable(lhs_value), rhs_value), 'expr']
+            if op in ['+=', '-=', '*=', '/=', '%=']:
+                return [aug_assign(Variable(lhs_value), op[0], rhs_value), 'expr']
 
         def get_expr_for_operand(self, combined_variable):
             """Gives out SymPy Codegen AST node
@@ -998,6 +1033,35 @@ if cin:
             if combined_variable[1] == 'boolean':
                     return true if combined_variable[0] == 'true' else false
 
+        def transform_null_stmt(self, node):
+            """Handles Null Statement and returns None"""
+            return none
+
+        def transform_while_stmt(self, node):
+            """Transformation function for handling while statement
+
+            Returns
+            =======
+
+            while statement : Codegen AST Node
+                contains the while statement node having condition and
+                statement block
+
+            """
+            children = node.get_children()
+
+            condition = self.transform(next(children))
+            statements = self.transform(next(children))
+
+            if isinstance(statements, list):
+                statement_block = CodeBlock(*statements)
+            else:
+                statement_block = CodeBlock(statements)
+
+            return While(condition, statement_block)
+
+
+
 else:
     class CCodeConverter():  # type: ignore
         def __init__(self, *args, **kwargs):
