@@ -19,7 +19,7 @@ from sympy import lambdify, Add
 from sympy.core.compatibility import iterable
 from sympy.utilities.decorator import doctest_depends_on
 
-numpy = import_module('numpy', __import__kwargs={'fromlist':['arange']})
+numpy = import_module('numpy', import_kwargs={'fromlist':['arange']})
 
 class Beam(object):
     """
@@ -72,7 +72,7 @@ class Beam(object):
          + Piecewise(((x - 2)**4, x - 2 > 0), (0, True))/4)/(E*I)
     """
 
-    def __init__(self, length, elastic_modulus, second_moment, variable=Symbol('x'), base_char='C'):
+    def __init__(self, length, elastic_modulus, second_moment, area=Symbol('A'), variable=Symbol('x'), base_char='C'):
         """Initializes the class.
 
         Parameters
@@ -98,6 +98,9 @@ class Beam(object):
             bending axis of the beam. The second moment of area will be computed
             from the shape object internally.
 
+        area : Symbol/float
+            Represents the cross-section area of beam
+
         variable : Symbol, optional
             A Symbol object that will be used as the variable along the beam
             while representing the load, shear, moment, slope and deflection
@@ -119,6 +122,7 @@ class Beam(object):
         self._base_char = base_char
         self._boundary_conditions = {'deflection': [], 'slope': []}
         self._load = 0
+        self._area = area
         self._applied_supports = []
         self._support_as_loads = []
         self._applied_loads = []
@@ -145,6 +149,15 @@ class Beam(object):
     def length(self, l):
         self._length = sympify(l)
 
+    @property
+    def area(self):
+        """Cross-sectional area of the Beam. """
+        return self._area
+
+    @area.setter
+    def area(self, a):
+        self._area = sympify(a)
+
     @property
     def variable(self):
         """
@@ -158,7 +171,7 @@ class Beam(object):
 
         >>> from sympy.physics.continuum_mechanics.beam import Beam
         >>> from sympy import symbols
-        >>> E, I = symbols('E, I')
+        >>> E, I, A = symbols('E, I, A')
         >>> x, y, z = symbols('x, y, z')
         >>> b = Beam(4, E, I)
         >>> b.variable
@@ -166,7 +179,7 @@ class Beam(object):
         >>> b.variable = y
         >>> b.variable
         y
-        >>> b = Beam(4, E, I, z)
+        >>> b = Beam(4, E, I, A, z)
         >>> b.variable
         z
         """
@@ -388,7 +401,17 @@ class Beam(object):
         Parameters
         ==========
         value : Sympifyable
-            The magnitude of an applied load.
+            The value inserted should have the units [Force/(Distance**(n+1)]
+            where n is the order of applied load.
+            Units for applied loads:
+
+               - For moments, unit = kN*m
+               - For point loads, unit = kN
+               - For constant distributed load, unit = kN/m
+               - For ramp loads, unit = kN/m/m
+               - For parabolic ramp loads, unit = kN/m/m/m
+               - ... so on.
+
         start : Sympifyable
             The starting point of the applied load. For point moments and
             point forces this is the location of application.
@@ -443,10 +466,10 @@ class Beam(object):
             # NOTE : A Taylor series can be used to define the summation of
             # singularity functions that subtract from the load past the end
             # point such that it evaluates to zero past 'end'.
-            f = value * x**order
+            f = value*x**order
             for i in range(0, order + 1):
                 self._load -= (f.diff(x, i).subs(x, end - start) *
-                               SingularityFunction(x, end, i) / factorial(i))
+                               SingularityFunction(x, end, i)/factorial(i))
 
     def remove_load(self, value, start, order, end=None):
         """
@@ -519,10 +542,10 @@ class Beam(object):
             # NOTE : A Taylor series can be used to define the summation of
             # singularity functions that subtract from the load past the end
             # point such that it evaluates to zero past 'end'.
-            f = value * x**order
+            f = value*x**order
             for i in range(0, order + 1):
                 self._load += (f.diff(x, i).subs(x, end - start) *
-                               SingularityFunction(x, end, i) / factorial(i))
+                               SingularityFunction(x, end, i)/factorial(i))
 
     @property
     def load(self):
@@ -731,7 +754,7 @@ class Beam(object):
         being positive.
 
         >>> from sympy.physics.continuum_mechanics.beam import Beam
-        >>> from sympy import symbols, linsolve, limit
+        >>> from sympy import symbols
         >>> E, I = symbols('E, I')
         >>> R1, R2 = symbols('R1, R2')
         >>> b = Beam(30, E, I)
@@ -1217,6 +1240,13 @@ class Beam(object):
         else:
             return None
 
+    def shear_stress(self):
+        """
+        Returns an expression representing the Shear Stress
+        curve of the Beam object.
+        """
+        return self.shear_force()/self._area
+
     def plot_shear_force(self, subs=None):
         """
 
@@ -1483,7 +1513,6 @@ class Beam(object):
 
             >>> from sympy.physics.continuum_mechanics.beam import Beam
             >>> from sympy import symbols
-            >>> from sympy.plotting import PlotGrid
             >>> R1, R2 = symbols('R1, R2')
             >>> b = Beam(8, 200*(10**9), 400*(10**-6))
             >>> b.apply_load(5000, 2, -1)
@@ -1526,7 +1555,18 @@ class Beam(object):
 
     @doctest_depends_on(modules=('numpy',))
     def draw(self, pictorial=True):
-        """Returns a plot object representing the beam diagram of the beam.
+        """
+        Returns a plot object representing the beam diagram of the beam.
+
+        .. note::
+            The user must be careful while entering load values.
+            The draw function assumes a sign convention which is used
+            for plotting loads.
+            Given a right handed coordinate system with XYZ coordinates,
+            the beam's length is assumed to be along the positive X axis.
+            The draw function recognizes positve loads(with n>-2) as loads
+            acting along negative Y direction and positve moments acting
+            along positive Z direction.
 
         Parameters
         ==========
@@ -1558,12 +1598,15 @@ class Beam(object):
             >>> b.apply_support(50, "pin")
             >>> b.apply_support(0, "fixed")
             >>> b.apply_support(20, "roller")
-            >>> b.draw()
+            >>> p = b.draw()
+            >>> p
             Plot object containing:
-            [0]: cartesian line: 25*SingularityFunction(x, 5, 0)
-            - 25*SingularityFunction(x, 23, 0) + SingularityFunction(x, 30, 1)
-            - 20*SingularityFunction(x, 50, 0) - SingularityFunction(x, 50, 1)
-            + 5 for x over (0.0, 50.0)
+            [0]: cartesian line: 25*SingularityFunction(x, 5, 0) - 25*SingularityFunction(x, 23, 0)
+            + SingularityFunction(x, 30, 1) - 20*SingularityFunction(x, 50, 0)
+            - SingularityFunction(x, 50, 1) + 5 for x over (0.0, 50.0)
+            [1]: cartesian line: 5 for x over (0.0, 50.0)
+            >>> p.show()
+
         """
         if not numpy:
             raise ImportError("To use this function numpy module is required")
@@ -1584,15 +1627,15 @@ class Beam(object):
 
         rectangles = []
         rectangles.append({'xy':(0, 0), 'width':length, 'height': height, 'facecolor':"brown"})
-        annotations, markers, load_eq, fill = self._draw_load(pictorial, length, l)
+        annotations, markers, load_eq,load_eq1, fill = self._draw_load(pictorial, length, l)
         support_markers, support_rectangles = self._draw_supports(length, l)
 
         rectangles += support_rectangles
         markers += support_markers
 
-        sing_plot = plot(height + load_eq, (x, 0, length),
+        sing_plot = plot(height + load_eq, height + load_eq1, (x, 0, length),
          xlim=(-height, length + height), ylim=(-length, 1.25*length), annotations=annotations,
-          markers=markers, rectangles=rectangles, fill=fill, axis=False, show=False)
+          markers=markers, rectangles=rectangles, line_color='brown', fill=fill, axis=False, show=False)
 
         return sing_plot
 
@@ -1606,11 +1649,15 @@ class Beam(object):
         markers = []
         load_args = []
         scaled_load = 0
-        load_eq = 0
-        higher_order = False
+        load_args1 = []
+        scaled_load1 = 0
+        load_eq = 0     # For positive valued higher order loads
+        load_eq1 = 0    # For negative valued higher order loads
         fill = None
-
+        plus = 0        # For positive valued higher order loads
+        minus = 0       # For negative valued higher order loads
         for load in loads:
+
             # check if the position of load is in terms of the beam length.
             if l:
                 pos =  load[1].subs(l)
@@ -1626,51 +1673,89 @@ class Beam(object):
             # moment loads
             elif load[2] == -2:
                 if load[0].is_negative:
-                    markers.append({'args':[[pos], [height/2]], 'marker': r'$\circlearrowleft$', 'markersize':15})
-                else:
                     markers.append({'args':[[pos], [height/2]], 'marker': r'$\circlearrowright$', 'markersize':15})
+                else:
+                    markers.append({'args':[[pos], [height/2]], 'marker': r'$\circlearrowleft$', 'markersize':15})
             # higher order loads
             elif load[2] >= 0:
-                higher_order = True
+                # `fill` will be assigned only when higher order loads are present
+                value, start, order, end = load
+                # Positive loads have their seperate equations
+                if(value>0):
+                    plus = 1
                 # if pictorial is True we remake the load equation again with
                 # some constant magnitude values.
-                if pictorial:
-                    value, start, order, end = load
-                    value = 10**(1-order) if order > 0 else length/2
+                    if pictorial:
+                        value = 10**(1-order) if order > 0 else length/2
+                        scaled_load += value*SingularityFunction(x, start, order)
+                        if end:
+                            f2 = 10**(1-order)*x**order if order > 0 else length/2*x**order
+                            for i in range(0, order + 1):
+                                scaled_load -= (f2.diff(x, i).subs(x, end - start)*
+                                               SingularityFunction(x, end, i)/factorial(i))
 
-                    scaled_load += value*SingularityFunction(x, start, order)
-                    if end:
-                        f2 = 10**(1-order)*x**order if order > 0 else length/2*x**order
-                        for i in range(0, order + 1):
-                            scaled_load -= (f2.diff(x, i).subs(x, end - start)*
-                                           SingularityFunction(x, end, i) / factorial(i))
+                    if pictorial:
+                        if isinstance(scaled_load, Add):
+                            load_args = scaled_load.args
+                        else:
+                            # when the load equation consists of only a single term
+                            load_args = (scaled_load,)
+                        load_eq = [i.subs(l) for i in load_args]
+                    else:
+                        if isinstance(self.load, Add):
+                            load_args = self.load.args
+                        else:
+                            load_args = (self.load,)
+                        load_eq = [i.subs(l) for i in load_args if list(i.atoms(SingularityFunction))[0].args[2] >= 0]
+                    load_eq = Add(*load_eq)
 
-        # `fill` will be assigned only when higher order loads are present
-        if higher_order:
-            if pictorial:
-                if isinstance(scaled_load, Add):
-                    load_args = scaled_load.args
+                    # filling higher order loads with colour
+                    expr = height + load_eq.rewrite(Piecewise)
+                    y1 = lambdify(x, expr, 'numpy')
+
+                # For loads with negative value
                 else:
-                    # when the load equation consists of only a single term
-                    load_args = (scaled_load,)
-                load_eq = [i.subs(l) for i in load_args]
-            else:
-                if isinstance(self.load, Add):
-                    load_args = self.load.args
+                    minus = 1
+                    # if pictorial is True we remake the load equation again with
+                    # some constant magnitude values.
+                    if pictorial:
+                        value = 10**(1-order) if order > 0 else length/2
+                        scaled_load1 += value*SingularityFunction(x, start, order)
+                        if end:
+                            f2 = 10**(1-order)*x**order if order > 0 else length/2*x**order
+                            for i in range(0, order + 1):
+                                scaled_load1 -= (f2.diff(x, i).subs(x, end - start)*
+                                               SingularityFunction(x, end, i)/factorial(i))
+
+                    if pictorial:
+                        if isinstance(scaled_load1, Add):
+                            load_args1 = scaled_load1.args
+                        else:
+                            # when the load equation consists of only a single term
+                            load_args1 = (scaled_load1,)
+                        load_eq1 = [i.subs(l) for i in load_args1]
+                    else:
+                        if isinstance(self.load, Add):
+                            load_args1 = self.load.args1
+                        else:
+                            load_args1 = (self.load,)
+                        load_eq1 = [i.subs(l) for i in load_args if list(i.atoms(SingularityFunction))[0].args[2] >= 0]
+                    load_eq1 = -Add(*load_eq1)-height
+
+                    # filling higher order loads with colour
+                    expr = height + load_eq1.rewrite(Piecewise)
+                    y1_ = lambdify(x, expr, 'numpy')
+
+                y = numpy.arange(0, float(length), 0.001)
+                y2 = float(height)
+
+                if(plus == 1 and minus == 1):
+                    fill = {'x': y, 'y1': y1(y), 'y2': y1_(y), 'color':'darkkhaki'}
+                elif(plus == 1):
+                    fill = {'x': y, 'y1': y1(y), 'y2': y2, 'color':'darkkhaki'}
                 else:
-                    load_args = (self.load,)
-                load_eq = [i.subs(l) for i in load_args if list(i.atoms(SingularityFunction))[0].args[2] >= 0]
-
-            load_eq = Add(*load_eq)
-
-            # filling higher order loads with colour
-            y = numpy.arange(0, float(length), 0.001)
-            expr = height + load_eq.rewrite(Piecewise)
-            y1 = lambdify(x, expr, 'numpy')
-            y2 = float(height)
-            fill = {'x': y, 'y1': y1(y), 'y2': y2, 'color':'darkkhaki'}
-
-        return annotations, markers, load_eq, fill
+                    fill = {'x': y, 'y1': y1_(y), 'y2': y2 , 'color':'darkkhaki'}
+        return annotations, markers, load_eq, load_eq1, fill
 
 
     def _draw_supports(self, length, l):
@@ -1720,7 +1805,7 @@ class Beam3D(Beam):
     is restricted.
 
     >>> from sympy.physics.continuum_mechanics.beam import Beam3D
-    >>> from sympy import symbols, simplify, collect
+    >>> from sympy import symbols, simplify, collect, factor
     >>> l, E, G, I, A = symbols('l, E, G, I, A')
     >>> b = Beam3D(l, E, G, I, A)
     >>> x, q, m = symbols('x, q, m')
@@ -1733,9 +1818,9 @@ class Beam3D(Beam):
     >>> b.bc_slope = [(0, [0, 0, 0]), (l, [0, 0, 0])]
     >>> b.bc_deflection = [(0, [0, 0, 0]), (l, [0, 0, 0])]
     >>> b.solve_slope_deflection()
-    >>> b.slope()
-    [0, 0, x*(l*(-l*q + 3*l*(A*G*l*(l*q - 2*m) + 12*E*I*q)/(2*(A*G*l**2 + 12*E*I)) + 3*m)/6
-        + q*x**2/6 + x*(-l*(A*G*l*(l*q - 2*m) + 12*E*I*q)/(2*(A*G*l**2 + 12*E*I)) - m)/2)/(E*I)]
+    >>> factor(b.slope())
+    [0, 0, x*(-l + x)*(-A*G*l**3*q + 2*A*G*l**2*q*x - 12*E*I*l*q
+        - 72*E*I*m + 24*E*I*q*x)/(12*E*I*(A*G*l**2 + 12*E*I))]
     >>> dx, dy, dz = b.deflection()
     >>> dy = collect(simplify(dy), x)
     >>> dx == dz == 0
@@ -1782,7 +1867,7 @@ class Beam3D(Beam):
         """
         super(Beam3D, self).__init__(length, elastic_modulus, second_moment, variable)
         self.shear_modulus = shear_modulus
-        self.area = area
+        self._area = area
         self._load_vector = [0, 0, 0]
         self._moment_load_vector = [0, 0, 0]
         self._load_Singularity = [0, 0, 0]
@@ -2037,6 +2122,19 @@ class Beam3D(Beam):
         """
         return self.shear_force()[0]
 
+    def shear_stress(self):
+        """
+        Returns a list of three expressions which represents the shear stress
+        curve of the Beam object along all three axes.
+        """
+        return [self.shear_force()[0]/self._area, self.shear_force()[1]/self._area, self.shear_force()[2]/self._area]
+
+    def axial_stress(self):
+        """
+        Returns expression of Axial stress present inside the Beam object.
+        """
+        return self.axial_force()/self._area
+
     def bending_moment(self):
         """
         Returns a list of three expressions which represents the bending moment
@@ -2066,7 +2164,7 @@ class Beam3D(Beam):
             I_y, I_z = I[0], I[1]
         else:
             I_y = I_z = I
-        A = self.area
+        A = self._area
         load = self._load_vector
         moment = self._moment_load_vector
         defl = Function('defl')
