@@ -6,18 +6,16 @@ their modules. Instead of instantiating the classes directly, you should use
 the function ``homomorphism(from, to, matrix)`` to create homomorphism objects.
 """
 
-from __future__ import print_function, division
 
 from sympy.polys.agca.modules import (Module, FreeModule, QuotientModule,
     SubModule, SubQuotientModule)
 from sympy.polys.polyerrors import CoercionFailed
-from sympy.core.compatibility import range
 
 # The main computational task for module homomorphisms is kernels.
 # For this reason, the concrete classes are organised by domain module type.
 
 
-class ModuleHomomorphism(object):
+class ModuleHomomorphism:
     """
     Abstract base class for module homomoprhisms. Do not instantiate.
 
@@ -76,6 +74,9 @@ class ModuleHomomorphism(object):
         That is, if ``self`` is the homomorphism `\phi: M \to N`, then compute
         `ker(\phi) = \{x \in M | \phi(x) = 0\}`.  This is a submodule of `M`.
 
+        Examples
+        ========
+
         >>> from sympy import QQ
         >>> from sympy.abc import x
         >>> from sympy.polys.agca import homomorphism
@@ -95,6 +96,9 @@ class ModuleHomomorphism(object):
         That is, if ``self`` is the homomorphism `\phi: M \to N`, then compute
         `im(\phi) = \{\phi(x) | x \in M \}`.  This is a submodule of `N`.
 
+        Examples
+        ========
+
         >>> from sympy import QQ
         >>> from sympy.abc import x
         >>> from sympy.polys.agca import homomorphism
@@ -137,6 +141,9 @@ class ModuleHomomorphism(object):
 
         Here ``sm`` has to be a submodule of ``self.domain``.
 
+        Examples
+        ========
+
         >>> from sympy import QQ
         >>> from sympy.abc import x
         >>> from sympy.polys.agca import homomorphism
@@ -174,6 +181,9 @@ class ModuleHomomorphism(object):
         Here ``sm`` has to be a submodule of ``self.codomain`` containing the
         image.
 
+        Examples
+        ========
+
         >>> from sympy import QQ
         >>> from sympy.abc import x
         >>> from sympy.polys.agca import homomorphism
@@ -202,6 +212,9 @@ class ModuleHomomorphism(object):
 
         Here ``sm`` must be a submodule of ``self.kernel()``.
 
+        Examples
+        ========
+
         >>> from sympy import QQ
         >>> from sympy.abc import x
         >>> from sympy.polys.agca import homomorphism
@@ -230,6 +243,9 @@ class ModuleHomomorphism(object):
 
         Here ``sm`` must be a submodule of ``self.codomain``.
 
+        Examples
+        ========
+
         >>> from sympy import QQ
         >>> from sympy.abc import x
         >>> from sympy.polys.agca import homomorphism
@@ -304,14 +320,12 @@ class ModuleHomomorphism(object):
     # NOTE: _compose will never be called from rmul
     __rmul__ = __mul__
 
-    def __div__(self, oth):
+    def __truediv__(self, oth):
         try:
             return self._mul_scalar(1/self.ring.convert(oth))
         except CoercionFailed:
             return NotImplemented
 
-    __truediv__ = __div__
-
     def __add__(self, oth):
         if self._check_hom(oth):
             return self._add(oth)
@@ -329,6 +343,9 @@ class ModuleHomomorphism(object):
         That is, check if the elements of the domain are mapped to the same
         codomain element.
 
+        Examples
+        ========
+
         >>> from sympy import QQ
         >>> from sympy.abc import x
         >>> from sympy.polys.agca import homomorphism
@@ -349,6 +366,9 @@ class ModuleHomomorphism(object):
         That is, check if every element of the codomain has at least one
         preimage.
 
+        Examples
+        ========
+
         >>> from sympy import QQ
         >>> from sympy.abc import x
         >>> from sympy.polys.agca import homomorphism
@@ -369,6 +389,9 @@ class ModuleHomomorphism(object):
         That is, check if every element of the codomain has precisely one
         preimage. Equivalently, ``self`` is both injective and surjective.
 
+        Examples
+        ========
+
         >>> from sympy import QQ
         >>> from sympy.abc import x
         >>> from sympy.polys.agca import homomorphism
@@ -390,6 +413,9 @@ class ModuleHomomorphism(object):
         That is, check if every element of the domain is mapped to zero
         under self.
 
+        Examples
+        ========
+
         >>> from sympy import QQ
         >>> from sympy.abc import x
         >>> from sympy.polys.agca import homomorphism
