@@ -1,8 +1,6 @@
 """Real and complex root isolation and refinement algorithms. """
 
-from __future__ import print_function, division
 
-from sympy.core.compatibility import range
 from sympy.polys.densearith import (
     dup_neg, dup_rshift, dup_rem)
 from sympy.polys.densebasic import (
@@ -1658,7 +1656,7 @@ def dup_isolate_all_roots(f, K, eps=None, inf=None, sup=None, fast=False):
     else:
         raise NotImplementedError( "only trivial square-free polynomials are supported")
 
-class RealInterval(object):
+class RealInterval:
     """A fully qualified representation of a real isolation interval. """
 
     def __init__(self, data, f, dom):
@@ -1788,7 +1786,7 @@ class RealInterval(object):
         return self._inner_refine()
 
 
-class ComplexInterval(object):
+class ComplexInterval:
     """A fully qualified representation of a complex isolation interval.
     The printed form is shown as (ax, bx) x (ay, by) where (ax, ay)
     and (bx, by) are the coordinates of the southwest and northeast
@@ -1797,7 +1795,7 @@ class ComplexInterval(object):
     Examples
     ========
 
-    >>> from sympy import CRootOf, Rational, S
+    >>> from sympy import CRootOf, S
     >>> from sympy.abc import x
     >>> CRootOf.clear_cache()  # for doctest reproducibility
     >>> root = CRootOf(x**10 - 2*x + 3, 9)
