@@ -44,11 +44,10 @@ from sympy.polys.domains import FF, ZZ, QQ, EX
 from sympy.polys.rings import ring
 
 from sympy import S, I, sin
-from sympy.core.compatibility import long
 
 from sympy.abc import x
 
-from sympy.utilities.pytest import raises
+from sympy.testing.pytest import raises
 
 f_0, f_1, f_2, f_3, f_4, f_5, f_6 = [ f.to_dense() for f in f_polys() ]
 
@@ -213,7 +212,7 @@ def test_dmp_eval_in():
     assert dmp_eval_in(f_6, 7, 2, 3, ZZ) == dmp_swap(
         dmp_eval(dmp_swap(f_6, 0, 2, 3, ZZ), 7, 3, ZZ), 0, 1, 2, ZZ)
 
-    f = [[[long(45)]], [[]], [[]], [[long(-9)], [-1], [], [long(3), long(0), long(10), long(0)]]]
+    f = [[[int(45)]], [[]], [[]], [[int(-9)], [-1], [], [int(3), int(0), int(10), int(0)]]]
 
     assert dmp_eval_in(f, -2, 2, 2, ZZ) == \
         [[45], [], [], [-9, -1, 0, -44]]
