@@ -15,7 +15,7 @@ TODO:
 from __future__ import print_function, division
 
 from .pretty_symbology import hobj, vobj, xsym, xobj, pretty_use_unicode, is_combining
-from sympy.core.compatibility import unicode
+from sympy.utilities.exceptions import SymPyDeprecationWarning
 
 class stringPict(object):
     """An ASCII picture.
@@ -349,10 +349,7 @@ class stringPict(object):
         return super(stringPict, self).__hash__()
 
     def __str__(self):
-        return str.join('\n', self.picture)
-
-    def __unicode__(self):
-        return unicode.join(u'\n', self.picture)
+        return '\n'.join(self.picture)
 
     def __repr__(self):
         return "stringPict(%r,%d)" % ('\n'.join(self.picture), self.baseline)
@@ -388,7 +385,20 @@ class prettyForm(stringPict):
         """Initialize from stringPict and binding power."""
         stringPict.__init__(self, s, baseline)
         self.binding = binding
-        self.unicode = unicode or s
+        if unicode is not None:
+            SymPyDeprecationWarning(
+                feature="``unicode`` argument to ``prettyForm``",
+                useinstead="the ``s`` argument",
+                deprecated_since_version="1.7").warn()
+        self._unicode = unicode or s
+
+    @property
+    def unicode(self):
+        SymPyDeprecationWarning(
+            feature="``prettyForm.unicode`` attribute",
+            useinstead="``stringPrict.s`` attribute",
+            deprecated_since_version="1.7").warn()
+        return self._unicode
 
     # Note: code to handle subtraction is in _print_Add
 
@@ -410,7 +420,7 @@ class prettyForm(stringPict):
             result.append(arg)
         return prettyForm(binding=prettyForm.ADD, *stringPict.next(*result))
 
-    def __div__(self, den, slashed=False):
+    def __truediv__(self, den, slashed=False):
         """Make a pretty division; stacked or slashed.
         """
         if slashed:
@@ -429,15 +439,12 @@ class prettyForm(stringPict):
             stringPict.LINE,
             den))
 
-    def __truediv__(self, o):
-        return self.__div__(o)
-
     def __mul__(self, *others):
         """Make a pretty multiplication.
         Parentheses are needed around +, - and neg.
         """
         quantity = {
-            'degree': u"\N{DEGREE SIGN}"
+            'degree': "\N{DEGREE SIGN}"
         }
 
         if len(others) == 0:
