@@ -48,7 +48,6 @@ def _process_limits(func, limits):
     Examples
     ========
 
-    >>> from sympy import pi
     >>> from sympy.series.fourier import _process_limits as pari
     >>> from sympy.abc import x
     >>> pari(x**2, (x, -2, 2))
@@ -406,7 +405,7 @@ class FourierSeries(SeriesBase):
 
         return self.func(sfunc, self.args[1], (self.a0, an, bn))
 
-    def _eval_as_leading_term(self, x):
+    def _eval_as_leading_term(self, x, cdir=0):
         for t in self:
             if t is not S.Zero:
                 return t
@@ -583,34 +582,144 @@ class FiniteFourierSeries(FourierSeries):
 
 
 def fourier_series(f, limits=None, finite=True):
-    """Computes Fourier sine/cosine series expansion.
+    r"""Computes the Fourier trigonometric series expansion.
 
-    Returns a :class:`FourierSeries` object.
+    Explanation
+    ===========
+
+    Fourier trigonometric series of $f(x)$ over the interval $(a, b)$
+    is defined as:
+
+    .. math::
+        \frac{a_0}{2} + \sum_{n=1}^{\infty}
+        (a_n \cos(\frac{2n \pi x}{L}) + b_n \sin(\frac{2n \pi x}{L}))
+
+    where the coefficients are:
+
+    .. math::
+        L = b - a
+
+    .. math::
+        a_0 = \frac{2}{L} \int_{a}^{b}{f(x) dx}
+
+    .. math::
+        a_n = \frac{2}{L} \int_{a}^{b}{f(x) \cos(\frac{2n \pi x}{L}) dx}
+
+    .. math::
+        b_n = \frac{2}{L} \int_{a}^{b}{f(x) \sin(\frac{2n \pi x}{L}) dx}
+
+    The condition whether the function $f(x)$ given should be periodic
+    or not is more than necessary, because it is sufficient to consider
+    the series to be converging to $f(x)$ only in the given interval,
+    not throughout the whole real line.
+
+    This also brings a lot of ease for the computation because
+    you don't have to make $f(x)$ artificially periodic by
+    wrapping it with piecewise, modulo operations,
+    but you can shape the function to look like the desired periodic
+    function only in the interval $(a, b)$, and the computed series will
+    automatically become the series of the periodic version of $f(x)$.
+
+    This property is illustrated in the examples section below.
+
+    Parameters
+    ==========
+
+    limits : (sym, start, end), optional
+        *sym* denotes the symbol the series is computed with respect to.
+
+        *start* and *end* denotes the start and the end of the interval
+        where the fourier series converges to the given function.
+
+        Default range is specified as $-\pi$ and $\pi$.
+
+    Returns
+    =======
+
+    FourierSeries
+        A symbolic object representing the Fourier trigonometric series.
 
     Examples
     ========
 
-    >>> from sympy import fourier_series, pi, cos
-    >>> from sympy.abc import x
+    Computing the Fourier series of $f(x) = x^2$:
 
-    >>> s = fourier_series(x**2, (x, -pi, pi))
-    >>> s.truncate(n=3)
+    >>> from sympy import fourier_series, pi
+    >>> from sympy.abc import x
+    >>> f = x**2
+    >>> s = fourier_series(f, (x, -pi, pi))
+    >>> s1 = s.truncate(n=3)
+    >>> s1
     -4*cos(x) + cos(2*x) + pi**2/3
 
-    Shifting
+    Shifting of the Fourier series:
 
     >>> s.shift(1).truncate()
     -4*cos(x) + cos(2*x) + 1 + pi**2/3
     >>> s.shiftx(1).truncate()
     -4*cos(x + 1) + cos(2*x + 2) + pi**2/3
 
-    Scaling
+    Scaling of the Fourier series:
 
     >>> s.scale(2).truncate()
     -8*cos(x) + 2*cos(2*x) + 2*pi**2/3
     >>> s.scalex(2).truncate()
     -4*cos(2*x) + cos(4*x) + pi**2/3
 
+    Computing the Fourier series of $f(x) = x$:
+
+    This illustrates how truncating to the higher order gives better
+    convergence.
+
+    .. plot::
+        :context: reset
+        :format: doctest
+        :include-source: True
+
+        >>> from sympy import fourier_series, pi, plot
+        >>> from sympy.abc import x
+        >>> f = x
+        >>> s = fourier_series(f, (x, -pi, pi))
+        >>> s1 = s.truncate(n = 3)
+        >>> s2 = s.truncate(n = 5)
+        >>> s3 = s.truncate(n = 7)
+        >>> p = plot(f, s1, s2, s3, (x, -pi, pi), show=False, legend=True)
+
+        >>> p[0].line_color = (0, 0, 0)
+        >>> p[0].label = 'x'
+        >>> p[1].line_color = (0.7, 0.7, 0.7)
+        >>> p[1].label = 'n=3'
+        >>> p[2].line_color = (0.5, 0.5, 0.5)
+        >>> p[2].label = 'n=5'
+        >>> p[3].line_color = (0.3, 0.3, 0.3)
+        >>> p[3].label = 'n=7'
+
+        >>> p.show()
+
+    This illustrates how the series converges to different sawtooth
+    waves if the different ranges are specified.
+
+    .. plot::
+        :context: close-figs
+        :format: doctest
+        :include-source: True
+
+        >>> s1 = fourier_series(x, (x, -1, 1)).truncate(10)
+        >>> s2 = fourier_series(x, (x, -pi, pi)).truncate(10)
+        >>> s3 = fourier_series(x, (x, 0, 1)).truncate(10)
+        >>> p = plot(x, s1, s2, s3, (x, -5, 5), show=False, legend=True)
+
+        >>> p[0].line_color = (0, 0, 0)
+        >>> p[0].label = 'x'
+        >>> p[1].line_color = (0.7, 0.7, 0.7)
+        >>> p[1].label = '[-1, 1]'
+        >>> p[2].line_color = (0.5, 0.5, 0.5)
+        >>> p[2].label = '[-pi, pi]'
+        >>> p[3].line_color = (0.3, 0.3, 0.3)
+        >>> p[3].label = '[0, 1]'
+
+        >>> p.show()
+
     Notes
     =====
 
@@ -634,7 +743,7 @@ def fourier_series(f, limits=None, finite=True):
     References
     ==========
 
-    .. [1] mathworld.wolfram.com/FourierSeries.html
+    .. [1] https://mathworld.wolfram.com/FourierSeries.html
     """
     f = sympify(f)
 
@@ -651,16 +760,18 @@ def fourier_series(f, limits=None, finite=True):
             return FiniteFourierSeries(f, limits, res_f)
 
     n = Dummy('n')
-    neg_f = f.subs(x, -x)
-    if f == neg_f:
-        a0, an = fourier_cos_seq(f, limits, n)
-        bn = SeqFormula(0, (1, oo))
-    elif f == -neg_f:
-        a0 = S.Zero
-        an = SeqFormula(0, (1, oo))
-        bn = fourier_sin_seq(f, limits, n)
-    else:
-        a0, an = fourier_cos_seq(f, limits, n)
-        bn = fourier_sin_seq(f, limits, n)
-
+    center = (limits[1] + limits[2]) / 2
+    if center.is_zero:
+        neg_f = f.subs(x, -x)
+        if f == neg_f:
+            a0, an = fourier_cos_seq(f, limits, n)
+            bn = SeqFormula(0, (1, oo))
+            return FourierSeries(f, limits, (a0, an, bn))
+        elif f == -neg_f:
+            a0 = S.Zero
+            an = SeqFormula(0, (1, oo))
+            bn = fourier_sin_seq(f, limits, n)
+            return FourierSeries(f, limits, (a0, an, bn))
+    a0, an = fourier_cos_seq(f, limits, n)
+    bn = fourier_sin_seq(f, limits, n)
     return FourierSeries(f, limits, (a0, an, bn))
