@@ -1,5 +1,3 @@
-from __future__ import print_function, division, absolute_import
-
 from typing import Callable, Dict, Optional, Tuple, Union
 
 from collections import OrderedDict
@@ -7,14 +5,13 @@ from distutils.errors import CompileError
 import os
 import re
 import subprocess
-import sys
 
 from .util import (
     find_binary_of_command, unique_list
 )
 
 
-class CompilerRunner(object):
+class CompilerRunner:
     """ CompilerRunner base class.
 
     Parameters
@@ -83,7 +80,7 @@ class CompilerRunner(object):
                 preferred_vendor = os.environ.get('SYMPY_COMPILER_VENDOR', None)
             self.compiler_name, self.compiler_binary, self.compiler_vendor = self.find_compiler(preferred_vendor)
             if self.compiler_binary is None:
-                raise ValueError("No compiler found (searched: {0})".format(', '.join(self.compiler_dict.values())))
+                raise ValueError("No compiler found (searched: {})".format(', '.join(self.compiler_dict.values())))
         self.define = define or []
         self.undef = undef or []
         self.include_dirs = include_dirs or []
@@ -178,18 +175,15 @@ class CompilerRunner(object):
                              stderr=subprocess.STDOUT,
                              env=env)
         comm = p.communicate()
-        if sys.version_info[0] == 2:
-            self.cmd_outerr = comm[0]
-        else:
-            try:
-                self.cmd_outerr = comm[0].decode('utf-8')
-            except UnicodeDecodeError:
-                self.cmd_outerr = comm[0].decode('iso-8859-1')  # win32
+        try:
+            self.cmd_outerr = comm[0].decode('utf-8')
+        except UnicodeDecodeError:
+            self.cmd_outerr = comm[0].decode('iso-8859-1')  # win32
         self.cmd_returncode = p.returncode
 
         # Error handling
         if self.cmd_returncode != 0:
-            msg = "Error executing '{0}' in {1} (exited status {2}):\n {3}\n".format(
+            msg = "Error executing '{}' in {} (exited status {}):\n {}\n".format(
                 ' '.join(self.cmd()), self.cwd, str(self.cmd_returncode), self.cmd_outerr
             )
             raise CompileError(msg)
